import { serializeObjectKey, unserializeObjectKey } from './utils';
/**
 * Retrieves all query parameters from the current URL as an object.
 * Each key and value in the query string is parsed and added to the result object.
 *
 * @returns {object} An object containing the URL's query parameters, deserialized if necessary.
 */
export function getQueryParams() {
    const params = new URLSearchParams(location.search);
    const result = {};
    for (const [key, value] of params.entries())
        result[key] = value;
    return unserializeObjectKey(result);
}
/**
 * Updates the current URL with the provided object's properties as query parameters.
 * Serializes the object, flattens it if needed, and sets each key-value pair as a query parameter.
 *
 * @template T
 * @param {T} data - An object containing key-value pairs to set as query parameters.
 * @returns {void} Does not return a value.
 */
export function setQueryParams(data) {
    const params = new URLSearchParams();
    const flattedObj = serializeObjectKey(data);
    Object.entries(flattedObj).forEach(([key, value]) => params.append(key, value));
    const queries = `?${params.toString()}`;
    history.replaceState(Object.assign(Object.assign({}, history.state), { current: history.state.current + queries }), '', queries);
}
/**
 * Retrieves a specific query parameter value from the current URL by matching a given property name.
 * Searches for a key that includes the provided property string, returning its associated value if found.
 *
 * @param {string} property - The property name to look for in the query parameters.
 * @returns {unknown} The value associated with the provided property in the URL, if it exists.
 */
export function getQueryParamsByKey(property) {
    const params = new URLSearchParams(location.search);
    let paramsKey = '', paramsValue = '';
    params.forEach((value, key) => {
        if (key.replace(/(\!\!|\(\))/g, '') === property) {
            paramsKey = key;
            paramsValue = value;
        }
    });
    const result = unserializeObjectKey({ [paramsKey]: paramsValue });
    //@ts-ignore
    return result[property];
}
/**
 * Sets or updates a single query parameter in the current URL with a given key-value pair.
 * Serializes the key-value pair if necessary and updates the URL without reloading the page.
 *
 * @param {string} key - The name of the query parameter to set or update.
 * @param {unknown} value - The value of the query parameter to set.
 * @returns {void} Does not return a value.
 */
export function setQueryParamsByKey(key, value) {
    const params = new URLSearchParams(location.search);
    const flattedObj = serializeObjectKey({ [key]: value });
    Object.entries(flattedObj).forEach(([k, val]) => params.set(k, val));
    const queries = `?${params.toString()}`;
    history.replaceState(Object.assign(Object.assign({}, history.state), { current: history.state.current + queries }), '', queries);
}
