/**
 * A map to convert specific string values to `null` or `undefined`.
 * Used in deserialization to handle special string representations.
 *
 * @constant {Map<string, null|undefined>}
 */
const nullOrUndefinedConversions = new Map([
    ['null', null],
    ['undefined', undefined]
]);
/**
 * Deserializes a flat object with serialized keys into a nested object structure.
 * Processes keys that may include delimiters (e.g., ".", "[]") to form nested structures,
 * handling specific key patterns for types like numbers and booleans.
 *
 * @template T
 * @param {T} serialized - An object where keys represent nested paths or types (e.g., "key[0].prop!!").
 * @returns {object} A deserialized object with nested structure and parsed values.
 * @throws {Error} Throws an error if the argument is not an object.
 */
export function unserializeObjectKey(serialized) {
    const result = {};
    if (typeof serialized !== 'object')
        throw new Error('The Argument is invalid, please make sure you are passing an object as the argument.');
    Object.entries(serialized).forEach(([key, value]) => {
        const keys = key.split(/[\.\[\]]+/).filter(Boolean);
        let current = result;
        let indicatorWithUndefinedKey = '';
        const lastKeyElement = keys[keys.length - 1];
        if (lastKeyElement === '()' || lastKeyElement === '!!')
            indicatorWithUndefinedKey = keys.pop();
        const lastKeysIndexAfterPopping = keys.length - 1;
        keys.forEach((k, index) => {
            const isArray = index < lastKeysIndexAfterPopping && /^[\d]+$/.test(keys[index + 1]);
            if (index === lastKeysIndexAfterPopping) {
                current[denormalizePrimitiveKey(k)] = valueParserByKey(k + indicatorWithUndefinedKey, value);
            }
            else {
                if (!current[k]) {
                    current[k] = isArray ? [] : {};
                }
                // @ts-ignore
                current = current[k];
            }
        });
    });
    return result;
}
/**
 * Serializes a nested object into a flat object with keys representing the path to each value.
 * Supports special suffixes to encode primitive types (e.g., numbers as `key()` and booleans as `key!!`).
 *
 * @template T
 * @param {T} unserialized - An object with nested structure to be serialized into a flat structure.
 * @returns {object} A flat object where each key represents the path to a nested value.
 * @throws {Error} Throws an error if the argument is not an object.
 */
export function serializeObjectKey(unserialized) {
    const acc = {};
    if (typeof unserialized !== 'object')
        throw new Error('The Argument is invalid, please make sure you are passing an object as the argument.');
    const recursive = (data, parentKey = '') => {
        if (Array.isArray(data)) {
            data.forEach((item, index) => {
                const currentKey = parentKey ? `${parentKey}[${index}]` : '';
                recursive(item, currentKey);
            });
        }
        else if (typeof data === 'object' && data !== null) {
            Object.entries(data).forEach(([key, value]) => {
                const currentKey = parentKey ? `${parentKey}.${key}` : key;
                recursive(value, currentKey);
            });
        }
        else
            acc[normalizePrimitiveKey(parentKey, data)] = data;
    };
    recursive(unserialized);
    return acc;
}
/**
 * Normalizes a key by adding a suffix to represent primitive data types.
 * Adds `()` for numbers and `!!` for booleans, leaving other types unchanged.
 *
 * @param {string} key - The key to normalize based on its associated value type.
 * @param {unknown} value - The value associated with the key, used to determine the suffix.
 * @returns {string} The normalized key with a suffix representing its data type, if applicable.
 */
export function normalizePrimitiveKey(key, value) {
    switch (typeof value) {
        case 'number':
            return `${key}()`;
        case 'boolean':
            return `${key}!!`;
        default:
            return key;
    }
}
/**
 * Removes any type-indicating suffix from a key, such as `()` or `!!`.
 *
 * @param {string} key - The key to denormalize by removing type suffixes, if present.
 * @returns {string} The key without any type-indicating suffix.
 */
export function denormalizePrimitiveKey(key) {
    const suffixes = ['()', '!!'];
    for (const suffix of suffixes)
        if (key.endsWith(suffix))
            return key.slice(0, -suffix.length);
    return key;
}
/**
 * Parses a value based on a key that may include type indicators.
 * If the key indicates a primitive type (e.g., "number()" or "boolean!!"), it parses the value accordingly.
 * Otherwise, returns the value directly, with special handling for null or undefined conversions.
 *
 * @param {string} key - The key associated with the value, potentially containing a type indicator.
 * @param {string} value - The value to parse, typically a string.
 * @returns {unknown} The parsed value, converted to the indicated type if applicable.
 */
export function valueParserByKey(key, value) {
    if (key.endsWith('()') || key.endsWith('!!'))
        return JSON.parse(value);
    return nullOrUndefinedConversions.has(value) ? nullOrUndefinedConversions.get(value) : value;
}
